@@ -37,7 +37,7 @@ typedef struct Efreet_Xml Efreet_Xml;
  */
 struct Efreet_Xml
 {
-    char *text;                         /**< The XML text for this node */
+    const char *text;                   /**< The XML text for this node */
     const char *tag;                    /**< The tag for this node */
 
     Efreet_Xml_Attribute **attributes;  /**< The attributes for this node */
@@ -46,7 +46,7 @@ struct Efreet_Xml
 };
 
 int efreet_xml_init(void);
-int efreet_xml_shutdown(void);
+void efreet_xml_shutdown(void);
 
 Efreet_Xml *efreet_xml_new(const char *file);
 void efreet_xml_del(Efreet_Xml *xml);
